//  Demo 27: ThreadPracticeG.java
//
//   Another (More Complex) Java Application
//
//   This continues the self animation Thread 
//   approach to animating objects.
//   
//   Uses the RunnableSquare class.
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999

// Very similar to ThreadPracticeC.java, except that we use self
// animating RunnableSquare objects rather than MovingSquare
// Thread objects.  Also, as an added bonus, we can control the
// speed of the animation of each individual RunnableSquare.

// Threads:
//
//   1. Main Thread -- this application.  Every running Java program
//        contains at least one Thread, the Thread main() runs on.
//   2. A separate Thread for each moving Square, as in Thread
//        PracticeB.java and ThreadPracticeC.java
      
import java.awt.*;
import java.awt.event.*;
import CloseAndExit;
import RunnableSquare;

public class ThreadPracticeG extends Frame 
                             implements ActionListener {

    RunnableSquare rs;  // A single RunnableSquare object.
                        // Each time we launch a new moving
                        // Square, we will reassign rst to
                        // the new object.

    Thread rsThread;  // the Thread rs will run on.  It too
                       // is reassigned each time we launch
                       // a new moving Square

    Image imageBuffer;    // for graphics double buffering as usual
 
    // for new MoveableSquare objects
    Label xLabel, yLabel,    // Labels for x, y components of 
          sizeLabel,         // position, size and color
          colorLabel;

    TextField xInput,        // TextFields for same
              yInput,
              sizeInput,
              colorInput;

    // for new MoveableSquare objects 
    Label rxLabel, ryLabel,  // Labels for x, y components of motion 
          dLabel;            // vector, and distance

    TextField rxInput,       // TextFields for same
              ryInput,
              dInput;

    // for animation speed
    Label pauseTimeLabel;
    TextField pauseTimeInput;

    Button launchButton;     // launch a new MoveableSquare

    int x, y, size;  // x and y components of new MoveableSquare 
                     // position, plus size of new MoveableSquare

    Color color;     // Color of new MoveableSquare

    int rx, ry, d;   // x and y components of motion vector, plus
                     // distance, in units of the motion vector
                     // length

    int pauseTime;   // pauseTime of animation, in milliseconds.
                     // The larger this is, the slower the
                     // animation runs.

    // Constructor: builds Frame and initializes variables.
    public ThreadPracticeG() {

       setLayout(new FlowLayout());

       // for new Square objects
       xLabel = new Label(" x Coord ");
       xInput = new TextField(4);
       xInput.setText("50");
       add(xLabel);  add(xInput);

       yLabel = new Label(" y Coord ");
       yInput = new TextField(4);
       yInput.setText("100");
       add(yLabel);  add(yInput);

       sizeLabel = new Label(" Size ");
       sizeInput = new TextField(4);
       sizeInput.setText("25");
       add(sizeLabel);  add(sizeInput);

       colorLabel = new Label("Color");
       colorInput = new TextField(5);
       colorInput.setText("blue");
       add(colorLabel);  add(colorInput);

       // for new MoveableSquare objects
       rxLabel = new Label(" x Motion ");
       rxInput = new TextField(3);
       rxInput.setText("1");
       add(rxLabel);  add(rxInput);

       ryLabel = new Label(" y Motion ");
       ryInput = new TextField(3);
       ryInput.setText("1");
       add(ryLabel);  add(ryInput);

       dLabel = new Label(" Distance ");
       dInput = new TextField(4);
       dInput.setText("200");
       add(dLabel);  add(dInput);

       // for animation speed
       pauseTimeLabel = new Label(" Pause Time ");
       pauseTimeInput = new TextField(4);
       pauseTimeInput.setText("20");
       add(pauseTimeLabel);   add(pauseTimeInput);

       launchButton = new Button("Launch");
       launchButton.addActionListener(this);
       add(launchButton);

       x = 50;  y = 100; size = 25; color = Color.blue;
       rx = 1; ry = 1;   d = 200;

       pauseTime = 20;
 
       addWindowListener(new CloseAndExit());   // as usual

    } // Constructor ThreadPracticeG()

    public void actionPerformed(ActionEvent evt) {

      String colorString;

      try {
        x = Integer.parseInt(xInput.getText());
      }
      catch (NumberFormatException e) {
        xInput.setText(Integer.toString(x)); // re-establish former
                                             // value
      }
 
      try {
        y = Integer.parseInt(yInput.getText());
      }
      catch (NumberFormatException e) {
        yInput.setText(Integer.toString(y)); // re-establish former
                                             // value
      }

      try {
        size = Integer.parseInt(sizeInput.getText());
      }
      catch (NumberFormatException e) {
        sizeInput.setText(Integer.toString(size)); 
                                      // re-establish former value
      }

      colorString = colorInput.getText();
      // always use the equals() method of the String class to
      // compare Strings for equality.  The == operator does object
      // comparison, that is it compares addresses, not content.
      if (colorString.equals("red"))
        color = Color.red;
      else if (colorString.equals("white"))
        color = Color.white;
      else if (colorString.equals("blue"))
        color = Color.blue;
      else if (colorString.equals("green"))
        color = Color.green;
      else { // restore default
        color = Color.blue;
        colorInput.setText("blue");
      }

      try {
        rx = Integer.parseInt(rxInput.getText());
      }
      catch (NumberFormatException e) {
        rxInput.setText(Integer.toString(rx)); // re-establish
                                               // former value
      }

      try {
        ry = Integer.parseInt(ryInput.getText());
      }
      catch (NumberFormatException e) {
        ryInput.setText(Integer.toString(ry)); // re-establish
                                               // former value
      }

      try {
        d = Integer.parseInt(dInput.getText());
      }
      catch (NumberFormatException e) {
        dInput.setText(Integer.toString(d)); // re-establish former
                                             // value
      }

      try {
        pauseTime = Integer.parseInt(pauseTimeInput.getText());
      }
      catch (NumberFormatException e) {
        pauseTimeInput.setText(Integer.toString(pauseTime));
                                             // re-establish former
                                             // value
      }

      rs = new RunnableSquare(x, y, size, rx, ry, d, color,
                              this, imageBuffer, pauseTime);

      // create and launch the Thread rs on which rs will run
      rsThread = new Thread(rs);
      rsThread.start();
      
   } // actionPerformed()

   // init(): creates the Image buffer object.  Remember, we can't
   //         do this in our Constructor, because we actually have
   //         to exist before the createImage method will work.
   //         We have to remember to call this method from main(),
   //         or we will get the dreaded "pointer reference to
   //         null" Exception.
   public void init() {

       imageBuffer = createImage(400, 400);

       while(imageBuffer == null) {

          try {
             Thread.sleep(50);  // 50 millisecond pause
          }
          catch(InterruptedException e) {
             // do nothing
          }

       }

   } // init()

   public static void main(String[ ] args) {

       ThreadPracticeG tpg = new ThreadPracticeG();
       tpg.setSize(400, 400);
       tpg.setVisible(true);
       tpg.init();  // don't forget to call this!

   } // main()

} // class ThreadPracticeG